/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */



#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <GL/glut.h>

#include "core.h"
#include "draw.h"
#include "drawmenu.h"
#include "drawworld.h"
#include "display.h"
#include "interface.h"
#include "jomar3D.h"
#include "scores.h"
#include "sound.h"
#include "utils.h"

#ifndef LANGDIR
#  define LANGDIR DATADIR"/lang"
#endif

static Martian l_mart[NMARTIANS];                     /* Storage of all infos about Martians */
static Lobster l_lobster = LOBSTER_STATIC_INIT;       /* All infos about the lobster */

static char* l_preffile;
static char* l_qualitytext = NULL;      /* name of the current quality set */

#define MAXLANG 10
#define NTEXT 56
char*         l_text[MAXLANG][NTEXT];   /* pointer to each line of text of each language file */
static int    l_nlang;                  /* number of languages listed */
static char*  l_lang[MAXLANG];          /* name of each lang, "unavailable" if true */
static int    l_uselangnb;              /* number of the current lang */

#define MAXSKIN 10
static int    l_nskin;                  /* number of skins listed */
static char*  l_skin[MAXSKIN];          /* name of each available skin */
static int    l_useskinnb;              /* number of the current skin */

static int l_bomb;                      /* Whether there is a bomb launched */
static int l_lostflag;                  /* Whether we have lost by exploding */
static int l_action;                    /* current action */
static unsigned char l_nextaction;      /* next move of the lobster */
static int l_lastaction;                /* useful for the crash sound in the walls */

static int l_booststate;                /* Current state of the booster capability */

#define terrain_check(x,y,v) ((l_terrain[(x)][(y)] == (v)) ? 1 : 0)
#define terrain_lobster(x,y) \
  ((terrain_check ((x),(y),T_LOBSTER_BACKWARD)  || \
    terrain_check ((x),(y),T_LOBSTER_FORWARD)   || \
    terrain_check ((x),(y),T_LOBSTER_LEFT)      || \
    terrain_check ((x),(y),T_LOBSTER_RIGHT)) ? 1 : 0)
#define terrain_martian(x,y) \
  ((terrain_check ((x),(y),T_MARTIAN_BACKWARD)  || \
    terrain_check ((x),(y),T_MARTIAN_FORWARD)   || \
    terrain_check ((x),(y),T_MARTIAN_LEFT)      || \
    terrain_check ((x),(y),T_MARTIAN_RIGHT)) ? 1 : 0)

static int l_terrain[T_X][T_Y]; /* = 0      : empty
                                 * = T_H    : wall
                                 * = 1-4    : lobster : 1: >y+      2: >x+       3: >y-       4: >x-
                                 * = 11-14  : martian : 11: >y+       12: >x+       13: >y-       14: >x-
                                 */

static int l_winuserdef = 0;    /* whether we should set a size for the first window */

static int l_gamestarttime;     /* time at which the game started */
static int l_gametime;          /* time of the finished game (0 if playing) */

static int l_martleft;          /* remaining Martians */

static int l_martsteps;         /* Number of steps already done by the Martians */

static int l_gamestate = 0;     /* Flags describing the current state of the game */

/* Maximum line length (for fgets) */
#define MAXLINE 1024


/* Say if we have lost */
int
core_getLost (void)
{
  return l_lostflag;
}

/* give the current state flag of jomar3D */
int
core_getState (void)
{
  return l_gamestate;
}

/* Add a state */
void
core_addState (int state)
{
  l_gamestate |= state;
}

/* Suppress a state */
void
core_deleteState (int state)
{
  l_gamestate &= ~state;
}

/* Inverse a state */
void
core_invertState (int state)
{
  l_gamestate ^= state;
}

/* return a pointer to the text whom number is given or NULL if the text is not
 * found */
const char *
core_getTextPointer (int number)
{
  const char* ptr;
  static const char* nil = "nil";
  
  ptr = l_text[l_uselangnb][number];
  if (!ptr)
  {
#if (DEBUG) == 2
    debugmsg (("WARNING: non-loaded string %d\n", number));
#endif
    ptr = nil;
  }
  
  return ptr;
}

/* Give the time elapsed since the beginning of the game */
int
core_getGameDuration (void)
{
  if (l_martleft == 0)
    return l_gametime;
  else
    return glutGet(GLUT_ELAPSED_TIME) - l_gamestarttime;
}

/* Give the number of steps that have been done by the poor Martians */
int
core_getMartianSteps (void)
{
  return l_martsteps;
}

/** Get a Martian
 * \param id a martian ID. This must be in range 0, NMARTIANS.
 * \returns a pointer to the Martian struct at the givent ID or NULL if the ID
 *          is out of range
 */
Martian *
core_getMartian (unsigned int id)
{
  return (id < NMARTIANS) ? &l_mart[id] : NULL;
}

int
core_getMartiansLeft (void)
{
  return l_martleft;
}

/** Get the lobster
 * \returns a pointer to the Lobster struct
 */
Lobster *
core_getLobster (void)
{
  return &l_lobster;
}

/* Give the current booster state */
int
core_getBoostState (void)
{
  return l_booststate;
}

/* stop the crazy speed of the boost */
void
core_stopBoost (int dummy)
{
  (void) dummy;
  l_lobster.steptime = LM_S;
#if (USESOUND)
  sound_stopChannel (BOOST);
#endif
}

/* Initialize the bosst process */
void
core_startBoost (void)
{
  if (l_booststate < 3)
  {
    l_booststate += 1;
    l_lobster.steptime = LM_B;
    drawworld_makeLobsterTail ();
    glutPostRedisplay ();
    glutTimerFunc (10. * 1000, core_stopBoost, 0);
#if (USESOUND)
    sound_mix (BOOST);
#endif
  }
}

/* find if the lobster can go where the player wants */
static int
core_tryMove (int actionhyp)
{
  int homposxt, homposyt, senshyp, dest;

  homposxt = l_lobster.pos.x;
  homposyt = l_lobster.pos.y;
  senshyp = l_terrain[l_lobster.pos.x][l_lobster.pos.y];
  switch (actionhyp)
  {
    case 2:                    /* turn right */
      senshyp++;
      break;
    case 3:                    /* turn left */
      senshyp--;
      break;
  }
  if (senshyp == 0)
    senshyp = 4;
  if (senshyp == 5)
    senshyp = 1;
  dest = 1;
  if (actionhyp == 4)
    dest = -2;
  switch (senshyp)
  {
    case 1:
      homposyt += dest;
      break;
    case 2:
      homposxt += dest;
      break;
    case 3:
      homposyt -= dest;
      break;
    case 4:
      homposxt -= dest;
      break;
  }

  if (homposxt == -1 || homposxt == T_X || homposyt == -1 || homposyt == T_Y)
  {
#if (USESOUND)
    if (actionhyp == l_lastaction && (actionhyp == 1 || actionhyp == 4))
      sound_mix (WALL);
#endif
    l_lobster.stepsize = T_C; /* don't disturb next action by trying to eat if unpossible to move */
    return 0;
  }
#if (USESOUND)
  if ((l_terrain[homposxt][homposyt] == T_WALL) &&
      (actionhyp == l_lastaction) && (actionhyp == 1 || actionhyp == 4))
    sound_mix (WALL);
#endif
  if (l_terrain[homposxt][homposyt] != T_EMPTY)
  {
    l_lobster.stepsize = T_C; /* don't disturb next action by trying to eat if unpossible to move */
    return 0;
  }

  l_terrain[l_lobster.pos.x][l_lobster.pos.y] = senshyp; /* sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x- */
  return 1;
}

/* React to a player action wish */
void
core_wantAction (int actiontobe, unsigned char character)
{
  if (!l_lobster.going)
  {
    if (core_tryMove (actiontobe) == 1)
      l_action = actiontobe;
    else
      l_action = 0;
  }

  l_nextaction = character;
}

/* Say it could now give a new bomb */
static void
core_allowNewBomb (int dummy)
{
  (void) dummy;
  l_bomb = FALSE;
}

/* Make the bomb explode */
static void
core_explodeBomb (int posxy)
{
  int i, j;
  /* Split the bomb position given on only one int by the timer */
  int posx = posxy / 100;
  int posy = posxy - posx * 100;
  
  if (!l_bomb || l_martleft == 0)
    return;
  
#if (USESOUND)
  sound_mix (BOOM);
#endif

  l_terrain[posx][posy] = T_EMPTY;

  glutTimerFunc (EXPLODE, core_allowNewBomb, 0);

  /* Loop to search for the lobster in the neighbourhood of the exploded bomb */
  for (i=posx-1; i<=posx+1; i++)
  {
    for (j=posy-1; j<=posy+1; j++)
    {
      if (i < 0 || i > T_X-1 || j < 0 || j > T_Y-1 )
        continue;
      if (terrain_lobster(i,j))
      {
        l_lostflag = TRUE;
        glutTimerFunc(EXPLODE, interface_changeMenu, MP);
        return;
      }
    }
  }
}

/* A Martian has dropped a bomb! Init this process. */
static void
core_launchBomb (int posx, int posy)
{
  int duration;
  if (l_bomb)
    return;
  l_bomb = TRUE;
  switch (l_terrain[posx][posy])
  {
    case T_MARTIAN_FORWARD:
      posy--;
      break;
    case T_MARTIAN_BACKWARD:
      posy++;
      break;
    case T_MARTIAN_RIGHT:
      posx--;
      break;
    case T_MARTIAN_LEFT:
      posx++;
      break;
  }
  l_terrain[posx][posy] = T_BOMB;

  /* determine a random duration (between 2 & 8 sec) */
  duration = (int) (MYRAND * 6000. + 2000.);

  draw_startBombDraw (posx, posy, duration);
  glutTimerFunc (duration, core_explodeBomb, posx*100 + posy);
}

/* wait a little and retry when we follow a Martian */
static void
core_moveBehindMartian (int dummy)
{
  (void) dummy;

  if (!core_tryMove (1))
    glutTimerFunc (100, core_moveBehindMartian, 1);
  else
    interface_onSKeyDown (l_nextaction, 0, 0);
}

/* find if the lobster can eat and do it */
void
core_tryEat (void)
{
  int dx = 0;
  int dy = 0;
  int i;
  int pass; /* first pass : try one case ahead, second, two cases */
  int lastspeed = l_lobster.steptime;
  
  if (core_getLost ())
    return;

/* don't allow trying to eat two times consecutively
 * or you could go very fast only by trying to eat ! */
  if ((l_lobster.steptime == LM_E && l_lobster.stepsize != T_C * 2.) ||
      l_lobster.steptime == LM_A)
    return;

  glutSetCursor (GLUT_CURSOR_DESTROY);
  glutTimerFunc (2000, display_hideCursor, 0);
  for (pass = 1; pass <= 2; pass++)
  {
    switch (l_terrain[l_lobster.pos.x][l_lobster.pos.y])
    {
      case T_LOBSTER_FORWARD:
        dy = pass;
        break;
      case T_LOBSTER_RIGHT:
        dx = pass;
        break;
      case T_LOBSTER_BACKWARD:
        dy = -pass;
        break;
      case T_LOBSTER_LEFT:
        dx = -pass;
        break;
    }
    /* Check we wouldn't do outside or quit */
    if (l_lobster.pos.x + dx < 0 || l_lobster.pos.x + dx > T_X - 1 ||
        l_lobster.pos.y + dy < 0 || l_lobster.pos.y + dy > T_Y - 1)
      return;
    /* Check there is no wall nor bomb */
    if (terrain_check(l_lobster.pos.x + dx, l_lobster.pos.y + dy, T_WALL) ||
        terrain_check(l_lobster.pos.x + dx, l_lobster.pos.y + dy, T_BOMB))
      return;
      
    debugmsg (("attack: %d\n", l_terrain[l_lobster.pos.x + dx][l_lobster.pos.y + dy]));

    if (terrain_martian (l_lobster.pos.x + dx, l_lobster.pos.y + dy))
    {
      /* Now there is no Martian more: eaten! */
      l_terrain[l_lobster.pos.x + dx][l_lobster.pos.y + dy] = T_EMPTY;
      /* Find which one was eaten */
      for (i = 0; i < NMARTIANS; i++)
      {
        if (l_mart[i].coords.x == l_lobster.pos.x + dx &&
            l_mart[i].coords.y == l_lobster.pos.y + dy &&
            l_mart[i].speed != 0.)
        {
#if (USESOUND)
          sound_mix (PICKUP);
#endif
          /* This Martian no more exists */
          l_mart[i].speed = 0.;
          /* One less to eat */
          l_martleft--;
          /* If only one left, mark it */
          if (l_martleft == 1)
            core_addState (JOMAR3D_LAST);
          /* If no Martian more, the game is finished */
          if (l_martleft == 0)
          {
            core_deleteState (JOMAR3D_LAST);
            core_addState (JOMAR3D_END);
            l_gametime = glutGet(GLUT_ELAPSED_TIME) - l_gamestarttime;
            glutTimerFunc (4000, scores_begin, l_gametime);
          }
          break;
        }
      }
      break;
    }
  }

  /* Now make the big move */
#if (USESOUND)
  sound_mix (TRYCATCH);
#endif
  l_lobster.steptime = LM_E;
  l_lobster.stepsize = T_C * 2.;
  l_lobster.going = FALSE;                    /* change values to make a big jump and do it */
  i = 0;
  if (l_lastaction == 1 && l_nextaction != 0)
    i = GLUT_KEY_UP;
  interface_onSKeyDown (GLUT_KEY_UP, 1, 1);
  if (i == 0 || l_nextaction == 0)
    interface_onKeyUp (GLUT_KEY_UP, 1, 1);
  if (lastspeed == LM_B)
    l_lobster.steptime = LM_B;
}

/* Start the move of the lobster */
void
core_startMotion (void)
{
  if (!l_lobster.going && l_action != 0)
  {
    /* slow down if we had tried to eat */
    if (l_lobster.steptime == LM_E && l_lobster.stepsize != T_C * 2.)
    {                   
      l_lobster.steptime = LM_A;
    }
    /* If no booster define the speed */
    else if (l_lobster.steptime != LM_B)
    {
      if (l_action == 3 || l_action == 2)
        l_lobster.steptime = LM_T;
      if (l_action == 1 || l_action == 4)
      {
        if (l_lobster.stepsize != T_C * 2.)
          l_lobster.steptime = LM_S;
      }
    }
    l_lobster.going = TRUE;
#if (USESOUND)
    if (core_checkState(JOMAR3D_PLAYMUSIC) && 
        l_lobster.steptime != LM_E)
      sound_mix (WALK);
#endif
    l_lobster.basetime = glutGet (GLUT_ELAPSED_TIME);
    draw_adjustVars (-1, l_action, &l_lobster);
    glutIdleFunc (draw_forceRedraw);
    if (l_lastaction == 0)
      glutTimerFunc (1000, display_hideCursor, 0);
  }
  l_lastaction = l_action;
  if (l_action == 0)
  {                       /* don't stop behind a Martian, retry later */
    if (!core_tryMove (1))
      core_moveBehindMartian (0);
  }
}

/* Stop to move */
void
core_stopMotion (void)
{
  l_nextaction = 0;
}

/* FIXME:
 * this function is quite long an repetitive, it should be reviewed.
 */
/* Manage all the Martians in function of the place of the lobster */
void
core_activeMartians (void)
{
  int i, j, k;
  float rtime;   /* remaining time before the Martian has finished to move */

  for (i = 0; i < NMARTIANS; i++)
  {
  retry:
    if (l_mart[i].speed == 0.)
      continue;
    if (l_mart[i].state)
    {
      /* moving */
      
      glutIdleFunc (draw_forceRedraw);

      rtime = l_mart[i].time + l_mart[i].speed - glutGet (GLUT_ELAPSED_TIME);
      
      debugmsg (("time remaining to move rtime %f\n", rtime));
      
      if (rtime <= 0)
      {
        l_mart[i].state = 0;
        l_mart[i].coordsgl.x = 0.;
        l_mart[i].coordsgl.y = 0.;
        l_mart[i].angle = l_mart[i].goangle;
        goto retry;
      }
      l_mart[i].angle = l_mart[i].angle - (l_mart[i].angle - l_mart[i].goangle) / 4.;
      if (l_mart[i].coordsgl.x > 0)
        l_mart[i].coordsgl.x = T_C / l_mart[i].speed * rtime;
      if (l_mart[i].coordsgl.x < 0)
        l_mart[i].coordsgl.x = -T_C / l_mart[i].speed * rtime;
      if (l_mart[i].coordsgl.y > 0)
        l_mart[i].coordsgl.y = T_C / l_mart[i].speed * rtime;
      if (l_mart[i].coordsgl.y < 0)
        l_mart[i].coordsgl.y = -T_C / l_mart[i].speed * rtime;
        
      debugmsg (("angle      %f %f\n", l_mart[i].angle, l_mart[i].goangle));
      debugmsg (("going pos  %f %f\n", l_mart[i].coordsgl.x, l_mart[i].coordsgl.y));
    }
    else
    {
      /* waiting for the lobster */
      float tbasem;
      int lookx, looky;

      tbasem = glutGet (GLUT_ELAPSED_TIME);

      /*       searching for the lobster under the Martian      M
                                                                x       \
                                                              x x x      = martview
                                                            x x x x x   /                  */
      for (j = -1; j >= -martview; j--)
      {
        for (k = -abs (j + 1); k <= abs (j + 1); k++)
        {
          lookx = l_mart[i].coords.x + k;
          looky = l_mart[i].coords.y + j;
          if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
            continue;
#if (DEBUG == 2)
          debugmsg (("looking at %d %d\n", lookx, looky));
#endif
          if (terrain_lobster (lookx, looky))
          {
            if (!terrain_check (l_mart[i].coords.x, l_mart[i].coords.y + 1, T_EMPTY) ||
                l_mart[i].coords.y == T_Y - 1)
            {
              if (!terrain_check (l_mart[i].coords.x + 1, l_mart[i].coords.y, T_EMPTY) ||
                  l_mart[i].coords.x == T_X - 1)
              {
                if (!terrain_check (l_mart[i].coords.x - 1, l_mart[i].coords.y, T_EMPTY) ||
                    l_mart[i].coords.y == 0)
                {
                  goto impossible;
                }
                goto left;
              }
              goto right;
            }
          up:
            l_mart[i].goangle = 180.;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_EMPTY;
            l_mart[i].coords.y += 1;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_MARTIAN_FORWARD;
            l_mart[i].coordsgl.x = 0.;
            l_mart[i].coordsgl.y = -T_C;
            l_mart[i].time = tbasem;
            l_mart[i].state = 1;
            goto startmove;
          }
        }
      }
/*          searching for the lobster at the left of the Martian  x
                                                                  x x
                                                                  x x x M
                                                                  x x
                                                                  x     */
      for (j = -1; j >= -martview; j--)
      {
        for (k = -abs (j + 1); k <= abs (j + 1); k++)
        {
          lookx = l_mart[i].coords.x + j;
          looky = l_mart[i].coords.y + k;
          if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
            continue;
#if (DEBUG == 2)
          debugmsg (("looking at %d %d\n", lookx, looky));
#endif
          if (terrain_lobster (lookx, looky))
          {
            if (!terrain_check (l_mart[i].coords.x + 1, l_mart[i].coords.y, T_EMPTY) ||
                l_mart[i].coords.x == T_X - 1)
            {
              if (!terrain_check (l_mart[i].coords.x, l_mart[i].coords.y + 1, T_EMPTY) ||
                  l_mart[i].coords.y == T_Y - 1)
              {
                if (!terrain_check (l_mart[i].coords.x, l_mart[i].coords.y - 1, T_EMPTY) ||
                    l_mart[i].coords.y == 0)
                {
                  goto impossible;
                }
                goto down;
              }
              goto up;
            }
          right:
            l_mart[i].goangle = 270.;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_EMPTY;
            l_mart[i].coords.x += 1;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_MARTIAN_RIGHT;
            l_mart[i].coordsgl.x = -T_C;
            l_mart[i].time = tbasem;
            l_mart[i].state = 1;
            goto startmove;
          }
        }
      }
      /*      searching for the lobster upwards the Martian  x x x x x  \
                                                               x x x     = martview
                                                                 x      /
                                                                 M            */
      for (j = 1; j <= martview; j++)
      {
        for (k = -abs (j - 1); k <= abs (j - 1); k++)
        {
          lookx = l_mart[i].coords.x + k;
          looky = l_mart[i].coords.y + j;
          if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
            continue;
#if (DEBUG == 2)
          debugmsg (("looking at %d %d\n", lookx, looky));
#endif
          if (terrain_lobster (lookx, looky))
          {
            if (!terrain_check (l_mart[i].coords.x, l_mart[i].coords.y - 1, T_EMPTY) ||
                l_mart[i].coords.y == 0)
            {
              if (!terrain_check (l_mart[i].coords.x + 1, l_mart[i].coords.y, T_EMPTY) ||
                  l_mart[i].coords.x == T_X - 1)
              {
                if (!terrain_check (l_mart[i].coords.x - 1, l_mart[i].coords.y, T_EMPTY) ||
                    l_mart[i].coords.x == 0)
                {
                  goto impossible;
                }
                goto left;
              }
              goto right;
            }
          down:
            l_mart[i].goangle = 0.;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_EMPTY;
            l_mart[i].coords.y -= 1;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_MARTIAN_BACKWARD;
            l_mart[i].coordsgl.x = 0.;
            l_mart[i].coordsgl.y = T_C;
            l_mart[i].time = tbasem;
            l_mart[i].state = 1;
            goto startmove;
          }
        }
      }
/*          searching for the lobster at the right of the Martian       x
                                                                      x x
                                                                  M x x x
                                                                      x x
                                                                        x     */
      for (j = 1; j <= martview; j++)
      {
        for (k = -abs (j - 1); k <= abs (j - 1); k++)
        {
          lookx = l_mart[i].coords.x + j;
          looky = l_mart[i].coords.y + k;
          if (lookx < 0 || lookx >= T_X || looky < 0 || looky >= T_Y)
            continue;
#if (DEBUG == 2)
          debugmsg (("looking at %d %d %d\n", lookx, looky, j));
#endif
          if (terrain_lobster (lookx, looky))
          {
            if (!terrain_check (l_mart[i].coords.x - 1, l_mart[i].coords.y, T_EMPTY) ||
                l_mart[i].coords.x == 0)
            {
              if (!terrain_check (l_mart[i].coords.x, l_mart[i].coords.y + 1, T_EMPTY) ||
                  l_mart[i].coords.y == T_Y - 1)
              {
                if (!terrain_check (l_mart[i].coords.x, l_mart[i].coords.y - 1,  T_EMPTY) ||
                    l_mart[i].coords.y == 0)
                {
                  goto impossible;
                }
                goto down;
              }
              goto up;
            }
          left:
            l_mart[i].goangle = 90.;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_EMPTY;
            l_mart[i].coords.x -= 1;
            l_terrain[l_mart[i].coords.x][l_mart[i].coords.y] = T_MARTIAN_LEFT;
            l_mart[i].coordsgl.x = T_C;
            l_mart[i].time = tbasem;
            l_mart[i].state = 1;
            goto startmove;
          }
        }
      }
    impossible:
      i = i;
    }
  }
  return;
startmove:
  if (MYRAND > 0.96 && !core_checkState (JOMAR3D_INMENU))
    core_launchBomb (l_mart[i].coords.x, l_mart[i].coords.y);
  l_martsteps += 1;
  goto retry;
}

/* calculate the values from which the lobster's drawing depends */
void
core_computeLobsterPosition (void)
{
  int dest;

  if (l_lobster.going)
  {
    int elapsed;            /* time elapsed since beginning of the move */

    elapsed = glutGet (GLUT_ELAPSED_TIME) - l_lobster.basetime;
    draw_adjustVars (elapsed, l_action, &l_lobster);
    if (elapsed > l_lobster.steptime)
    {
      l_lobster.going = FALSE;
      dest = 1;
      if (l_action == 4)
        dest = -1;
      switch (l_terrain[l_lobster.pos.x][l_lobster.pos.y])
      {
        /*  1: >y+    2: >x+    3: >y-    4: >x- */
        case 1:
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_EMPTY; 
          l_lobster.pos.y += dest;
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_LOBSTER_FORWARD;   
          break;
        case 2:
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_EMPTY; 
          l_lobster.pos.x += dest;
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_LOBSTER_RIGHT;  
          break;
        case 3:
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_EMPTY; 
          l_lobster.pos.y -= dest;
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_LOBSTER_BACKWARD;  
          break;
        case 4:
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_EMPTY; 
          l_lobster.pos.x -= dest;
          l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_LOBSTER_LEFT;   
          break;
        default:
          fprintf (stderr, "error by orientation of the lobster(1) %d",
                   l_terrain[l_lobster.pos.x][l_lobster.pos.y]);
          break;
      }

      /* don't turn two times when boosted otherwise the game isn't no more playable */
      if ((l_lastaction == 3 || l_lastaction == 2) && l_lobster.steptime == LM_B)
      {
        l_nextaction = 0;
      }

      l_lastaction = l_action;
      l_action = 0;
      l_lobster.stepsize = T_C;
      if (l_nextaction != 0)
      {
        interface_onSKeyDown (l_nextaction, 1, 1);
        core_computeLobsterPosition ();
      }
    }
  }
  else
  {
    glutIdleFunc (NULL);
    l_lastaction = 0;
#if (USESOUND)
    sound_stopChannel (WALK);
#endif
  }
}

/* place the wells */
static void
core_placeWells (void)
{
  int randx, randy, sens, ix, iy, i, j, k, k1, k2;

/*  loop on each case to set all empty (0) */
  for (i = 0; i < T_X; i++)
    for (j = 0; j < T_Y; j++)
      l_terrain[i][j] = T_EMPTY;

/* set the position of the lobster */
  l_terrain[l_lobster.pos.x][l_lobster.pos.y] = T_LOBSTER_FORWARD;   /* sens 1: >y+  sens 2: >x+  sens 3: >y-  sens 4: >x- */
  l_terrain[l_lobster.pos.x][l_lobster.pos.y - 1] = T_LOCKED;        /* don't put anything on the tail of the lobster */

/* loop on each wall to put on the map */
  for (i = 1; i <= WALLLONG; i++)
  {
    for (j = 0; j < NEWALL; j++)
    {
    deb:
      ix = iy = 0;
      
/*     randx = x position of the beginning of the wall
   randy = x position of the beginning of the wall
   sens  = orientation in french
            if sens = 1 -> right
            if sens = 2 -> top
            if sens = 3 -> left
            if sens = 4 -> bottom                */
      randx = (int) (MYRAND * (T_X - 1) + 0.5) + 0;
      randy = (int) (MYRAND * (T_Y - 1) + 0.5) + 0;
      sens = (int) (MYRAND * (4 - 1) + 0.5) + 1;
      
      if (sens == 1)
        ix = 1;
      if (sens == 2)
        iy = 1;
      if (sens == 3)
        ix = -1;
      if (sens == 4)
        iy = -1;
      
      if (randx + ix * i < 0    ||
          randx + ix * i >= T_X ||
          randy + iy * i < 0    ||
          randy + iy * i >= T_Y)
        goto deb;
      
/*       check that all the squares are empty or request another random */
      for (k = 0; k < i; k++)
        if (!terrain_check (randx + ix * k, randy + iy * k, T_EMPTY))
          goto deb;
      
/*       filling of the position of the well  */
      for (k = 0; k < i; k++)
        l_terrain[randx + ix * k][randy + iy * k] = T_WALL;
      
/*       lock the neighbour squares */
      for (k = 0; k < i; k++)
      {
        for (k1 = -1; k1 < 2; k1++)
        {
          for (k2 = -1; k2 < 2; k2++)
          {
            if (randx + ix * k + k1 >= 0 && randx + ix * k + k1 < T_X &&
                randy + iy * k + k2 >= 0 && randy + iy * k + k2 < T_Y &&
                terrain_check (randx + ix * k + k1, randy + iy * k + k2, T_EMPTY))
            {
              l_terrain[randx + ix * k + k1][randy + iy * k + k2] = T_LOCKED;
            }
          }
        }
      }
      
/*       fill the tabular saying where all the wells are */
      drawworld_rememberWall (i-1, j, randx, randy, sens);
    }
  }

/* turn all locked squares to empty */
  for (k1 = 0; k1 < T_X; k1++)
  {
    for (k2 = 0; k2 < T_Y; k2++)
    {
      if (terrain_check (k1, k2, T_LOCKED))
        l_terrain[k1][k2] = T_EMPTY;
    }
  }
}

/* place the Martians */
static void
core_placeMartians (void)
{
  int randx, randy, sens;       /* random position and direction of the Martian */
  int i;

  for (i = 0; i < NMARTIANS; i++)
  {
  occuped:
    randx = (int) (MYRAND * (T_X - 1) + 0.5) + 0;
    randy = (int) (MYRAND * (T_Y - 1) + 0.5) + 0;
    sens = (int) (MYRAND * (4 - 1) + 0.5) + 1;
    /*  1: >y+       2: >x+       3: >y-       4: >x- */
    if (terrain_check (randx, randy, T_EMPTY))
    {
      l_terrain[randx][randy] = sens + 10; /* deprecated but still used... */
      l_mart[i].coords.x = randx;
      l_mart[i].coords.y = randy;
      l_mart[i].angle = (sens - 1) * 90. + 180.;
    }
    else
    {
      goto occuped;
    }
  }

#if (DEBUG)
  {
    int j;

    for (j = T_Y - 1; j >= 0; j--)
    {
      for (i = 0; i < T_X; i++)
      {
        printf ("%3d", l_terrain[i][j]);
      }
      printf ("\n");
    }
  }
#endif
}

/* do the chronic work: calculate the frames done since last call and try a random to launch an ambient sound */
static void
core_doCron (int dummy)
{
  (void) dummy;

  draw_calculateFPS (0);

#if (USESOUND)
/* now, use a random to launch an ambient sound */
  if (MYRAND > 0.97)
  {
    sound_mix (CHOU);
  }
#endif

  glutTimerFunc (1000, core_doCron, 0);  /* don't forget to be called another time ! */

  debugmsg (("Martian is at  %i  of %i in x and  %i  of %i in y\n",
            l_mart[0].coords.x, T_X, l_mart[0].coords.y, T_Y));
}

/* (re)initialize the variables for a new game */
void
core_initGame (void)
{
  int i;

  core_deleteState (JOMAR3D_INMENU);
  l_action = 0;
  l_lobster.pos.x = 0;
  l_lobster.pos.y = 1;
  core_placeWells ();
  l_martleft = NMARTIANS;
  core_deleteState (JOMAR3D_END);
  l_martsteps = 0;
  draw_setView (1);
  draw_adjustVars (-1, 0, &l_lobster);
  l_lobster.going = FALSE;
  l_lobster.steptime = 0.;
  l_lobster.stepsize = T_C;
  l_booststate = 0;
  l_lostflag = FALSE;
  l_bomb = FALSE;
#if (USESOUND)
  sound_stopChannel (WALK);
#endif
/* set the speed of each Martian */
  for (i = 0; i < NMARTIANS; i++)
#if (DEBUG)
    /* speed down the martian when in debugging mode */
    l_mart[i].speed = 500.;
#else
    l_mart[i].speed = (0.2525 + i * 0.01) * 1000.;
#endif

  core_placeMartians ();
  draw_setGameAttributes ();
  l_gametime = 0;
  l_gamestarttime = glutGet (GLUT_ELAPSED_TIME);
  glutPostRedisplay ();
}

/* Change the current language */
void
core_setLanguage (int move)
{
  poor:
  l_uselangnb += move;
  
  if (l_uselangnb >= l_nlang)
    l_uselangnb = 0;
  else if (l_uselangnb < 0)
    l_uselangnb = l_nlang - 1;
  
  if (streq (l_lang[l_uselangnb], "unavailable"))
    goto poor;
  drawmenu_createLists (0);
  draw_redraw ();
  glutPostRedisplay ();
}

/* return the current lang name's pointer */
const char*
core_getLanguageName (void)
{
  return l_lang[l_uselangnb];
}

/* Change the current skin */
void
core_setSkin (int move)
{
  l_useskinnb += move;
  
  if (l_useskinnb >= l_nskin)
    l_useskinnb = 0;
  else if (l_useskinnb < 0)
    l_useskinnb = l_nskin - 1;
  
  draw_loadTextures (0);
  drawmenu_createLists (0);
  drawworld_createTerrainLists ();
  draw_redraw ();
  glutPostRedisplay ();
}

/* return the current skin name's pointer */
const char*
core_getSkinName (void)
{
  return l_skin[l_useskinnb];
}

/* automatically called routine on exit */
static void
core_cleanQuit (void)
{
  int i, j;
  
#if (USESOUND)
  sound_unloadAll ();
#endif
  free (l_preffile);
  free (l_qualitytext);
  for (i=0; i<MAXLANG; i++)
    for (j=0; j<NTEXT; j++)
      free (l_text[i][j]);

  for (i = 0; i < l_nskin; i++)
    free (l_skin[i]);
  for (i = 0; i < l_nlang; i++)
    free (l_lang[i]);
  for (i = 0; i < MAXNAMES; i++)
    free (scores_getName (i));

  draw_quit ();
  display_destroyDisplay ();

  printf ("See you soon!\n");
}

/* filter for the language files */
static int
core_languageFilter (const struct dirent *a)
{
  /*if (strstr (a->d_name + (strlen (a->d_name) - 4), ".jlg"))*/
  /*
   * may be a little slower than
   * if (streq (a->d_name + (strlen (a->d_name) - 4), ".jlg"))
   * but really more readable and updateable
   */
  const char* ext = getext (a->d_name);
  if (ext && streq (ext, "jlg"))
    return 1;
  return 0;
}

/* load all available language */
static void
core_loadLanguages (void)
{
  struct dirent **namelist;
  char rep[MAXLINE];
  char *pos;
  int i, j;
  int n;
  int nchar;
  FILE *file;
  char *ptr;

  /* Set all text pointers to NULL */
  for (i=0; i<MAXLANG; i++)
    for (j=0; j<NTEXT; j++)
      l_text[i][j] = NULL;

  /* List the given directory */
  l_nlang = myscandir (LANGDIR, &namelist, core_languageFilter);
  if (l_nlang == -1)
  {
    perror ("scandir");
    fprintf (stderr, "Error listing the language directory '"LANGDIR
                     "'. You will probably experience problems...\n");
    l_nlang = 0;
    return;
  }
  else
  {
    if (l_nlang > MAXLANG)
      l_nlang = MAXLANG;
    for (n = 0; n < l_nlang; n++)
    {
      /* cut the filename's extension so it's OK for printing */
      rmext (namelist[n]->d_name);

      /* Malloc and store the filename now without extension */
      l_lang[n] = malloc (strlen (namelist[n]->d_name) + 1);
      if (l_lang[n])
        sprintf (l_lang[n], "%s", namelist[n]->d_name);
      else
        perror ("failed to allocate memory");
    }
      
    debugmsg (("nlang: %d\n", l_nlang));
    /* load each language into malloc'ed memory */
    for (i = 0; i < l_nlang; i++)
    {
      ptr = malloc ((strlen (LANGDIR "/") + strlen (l_lang[i]) + 4 + 1));
      if (ptr)
        sprintf (ptr, "%s%s%s", LANGDIR "/", l_lang[i], ".jlg");
      else
      {
        perror ("failed to allocate memory");
        break;
      }
        
      file = fopen (ptr, "r");
      free (ptr);
      
      if (file == NULL)
      {
        perror ("fopen for read");
        fprintf (stderr, "Error opening a language file: %s!\n", l_lang[i]);
        free (l_lang[i]);
        l_lang[i] = mystrdup ("unavailable");
        continue;
      }
        
      for (j = 0; j < NTEXT; j++)
      {
        if (fgets (rep, MAXLINE, file) == NULL)
        {
          perror ("fgets");
          fprintf (stderr, "Error while reading a language: %s!\n", l_lang[i]);
          free (l_lang[i]);
          l_lang[i] = mystrdup ("unavailable");
          break;
        }
        pos = strstr (rep, "\\");
        nchar = pos - rep;
        if (pos == NULL)
          nchar = strlen (rep);
          
        l_text[i][j] = malloc (nchar);
        if (l_text[i][j])
        {
          strncpy (l_text[i][j], rep, nchar);
          *(l_text[i][j] + nchar - 1) = '\0';
        }
        else
          perror ("failed to allocate memory");
        
      }
      fclose (file);
    }
  }
  
  /* free dirent struct */
  i = l_nlang;
  while (i--)
    free (namelist[i]);
  free (namelist);
}

/* select the language to use reading the name stored in preffile */
static void
core_selectLanguage (void)
{
  char rep[MAXLINE];
  char *envlang;
  FILE *file;
  int i;

  /* set default value using the system language */

  envlang = getenv("LANG");
  
  /* try to find \p langname in the lang array and set it as default language */
  #define SETLANG(langname) \
    for (i = 0; i < l_nlang; i++) \
      if (streq (l_lang[i], langname)) \
        l_uselangnb = i
  
  /* search for english, default : no condition
   * cannot be an else statement : if the lang wanted isn't found ? */
  SETLANG ("english");
  /* search for french */
  if (!strncmp(envlang, "fr", 2))
    SETLANG ("français");
  /* search for german */
  else if (!strncmp(envlang, "de", 2))
    SETLANG ("deutsch");
  /* search for spanish */
  else if (!strncmp(envlang, "es", 2))
    SETLANG ("español");
  
  /* open & read */

  file = fopen (l_preffile, "r");
  if (file == NULL)
  {
    perror ("fopen for read");
    fprintf (stderr, "Error opening %s.\n", l_preffile);
    return;
  }
  else
  {
    errno = 0;
    
    if (fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL)
    {
      perror ("fgets");
      fprintf (stderr, "Error reading %s.\n", l_preffile);
      fclose (file);
      return;
    }
  }
  fclose (file);

  /* cut trailing LF */
  rep[strlen (rep) - 1] = '\0';
  
  debugmsg (("Read '%s' as language to use\n", rep));

  /* find the stored language in the list read from the directory */
  SETLANG (rep);
  
#undef SETLANG
}

/* Set a new text for the current quality name */
void
core_setQualityText (const char *text)
{
  free (l_qualitytext);
  l_qualitytext = malloc (strlen(text) +1);
  strcpy (l_qualitytext, text);
}

/* Initialize all what should be by the quality setting */
/* NOTE: Must be called the window open because it
 *       launches all the OpenGl definitions that are depending on it! */
static void
core_initQuality (void)
{
  int quality = Q_NORMAL;
  FILE *file;
  char rep[MAXLINE];

  file = fopen (l_preffile, "r");
  if (file == NULL)
  {
    perror ("fopen for read");
    fprintf (stderr, "Error opening %s, the settings storage file\n", l_preffile);
    draw_applyQuality (0, quality);
    return;
  }
  else
  {
    errno = 0;
    
    if (fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL)
    {
      perror ("fgets");
      fprintf (stderr, "Error reading %s.\n", l_preffile);
      draw_applyQuality (0, quality);
      fclose (file);
      return;
    }
  }
  fclose (file);

  if (streq (rep, "horrible\n"))
    quality = Q_HORRIBLE;
  if (streq (rep, "normal\n"))
    quality = Q_NORMAL;
  if (streq (rep, "fine\n"))
    quality = Q_FINE;

  draw_applyQuality (0, quality);
}

/* filter for the skin files */
static int
core_skinFilter (const struct dirent *a)
{
/*if (strstr (a->d_name, ".jsk") && a->d_type == DT_DIR) return 1; type of file unknown in the mingw32 so don't take care and hope...*/
  /*if (strstr (a->d_name + (strlen (a->d_name) - 4), ".jsk"))*/
  /*
   * may be a little slower than
   * if (streq (a->d_name + (strlen (a->d_name) - 4), ".jsk"))
   * but really more readable and updateable
   */
  const char* ext = getext (a->d_name);
  if (ext && streq (ext, "jsk"))
    return 1;
  return 0;
}

/* find all skins in the directory */
static void
core_findSkins (void)
{
  struct dirent **namelist;
  int n;

  l_nskin = myscandir (SKINDIR, &namelist, core_skinFilter);
  if (l_nskin == -1)
  {
    perror ("scandir");
    fprintf (stderr, "Error listing the skin directory '"SKINDIR
             "'. You will probably experience problems...\n");
    l_nskin = 0;
    return;
  }

  if (l_nskin > MAXSKIN)
    l_nskin = MAXSKIN;
  for (n = 0; n < l_nskin; n++)
  {
    /* cut the file name's extension so the names are good for printing */
    rmext (namelist[n]->d_name);

    l_skin[n] = malloc (strlen (namelist[n]->d_name) + 1);
    if (l_skin[n])
      sprintf (l_skin[n], "%s", namelist[n]->d_name);
    else
      perror ("failed to allocate memory");
    
    debugmsg (("%s\n", l_skin[n]));
  }

  /* free dirent struct */
  n = l_nskin;
  while (n--)
    free (namelist[n]);
  free (namelist);
  
  debugmsg (("number of skins %d\n", l_nskin));
}

/* select the skin to use reading jomar3Dpref */
static void
core_selectSkin (void)
{
  char rep[MAXLINE];
  FILE *file;
  int i;

  for (i = 0; i < l_nskin; i++)
    if (streq (l_skin[i], "summer"))
      l_useskinnb = i;

  file = fopen (l_preffile, "r");
  if (file == NULL)
  {
    perror ("fopen for read");
    fprintf (stderr, "Error opening %s.\n", l_preffile);
    return;
  }
  else
  {
    errno = 0;
    
    if (fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL)
    {
      perror ("fgets");
      fprintf (stderr, "Error reading %s.\n", l_preffile);
      fclose (file);
      return;
    }
  }
  fclose (file);
  
  rep[strlen (rep) - 1] = '\0';
  
  debugmsg (("%s\n", rep));
  
  for (i = 0; i < l_nskin; i++)
    if (streq (l_skin[i], rep))
      l_useskinnb = i;
}

#if (USESOUND)
/* Read the preference file and initialise the sound volume consequently */
void
core_initVolume (void)
{
  char rep[MAXLINE];
  FILE *file;

  /* open the preference file */
  file = fopen (l_preffile, "r");
  if (file == NULL)
  {
    perror ("fopen for read");
    fprintf (stderr, "Error opening %s.\n", l_preffile);
    return;
  }
  else
  {
    errno = 0;
    
    /* read eight records to have the good line */
    if (fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL ||
        fgets (rep, MAXLINE, file) == NULL)
    {
      perror ("fgets");
      fprintf (stderr, "Error reading %s.\n", l_preffile);
      fclose (file);
      return;
    }

    /* use the result of the last read */
    sound_changeVolume (S_SET, (float)atof(rep));

    /* get one more record to know if paused or playing */
    if (fgets (rep, MAXLINE, file) == NULL)
    {
      perror ("fgets");
      fprintf (stderr, "Error reading %s.\n", l_preffile);
      fclose (file);
      return;
    }

    if (streq (rep, "paused\n"))
      sound_pauseResume ();

    /* Now we have finished for that file */
    fclose (file);
  }
}
#endif

/* write the options to use into jomar3Dpref */
void
core_saveOptions (void)
{
  FILE *file;
  int i;
  
  debugmsg (("saving the options...\n"));

  /* Open the file */
  if (!(file = fopen (l_preffile, "w")))
  {
    perror ("fopen for write");
    fprintf (stderr, "Cannot open %s... no save of the options!\n", l_preffile);
    return;
  }

  /* Reset previous errors */
  errno = 0;

  /* Write */

  /* Language part */
  fprintf (file, "# language to use - possible: "LANGDIR"/*.jlg (last run were \"");
  for (i = 0; i < l_nlang; i++)
    if (!streq (l_lang[i], "unavailable"))
      fprintf (file, "%s, ", l_lang[i]);
  fprintf (file, "\b\b\" available)\n");
  fprintf (file, "%s\n", l_lang[l_uselangnb]);

  /* Quality */
  fprintf (file, "# quality to run with\n");
  fprintf (file, "%s\n", l_qualitytext);

  /* Skin */
  fprintf (file, "# look out (skin) - possible: "SKINDIR"/*.jsk (last run were \"");
  for (i = 0; i < l_nskin; i++)
    fprintf (file, "%s, ", l_skin[i]);
  fprintf (file, "\b\b\" available)\n");
  fprintf (file, "%s\n", l_skin[l_useskinnb]);

  /* Sound */
#if (USESOUND)
  fprintf (file, "# initial volume (%%)\n");
  fprintf (file, "%.1f\n", sound_getVolume ());
  if (core_checkState (JOMAR3D_PLAYMUSIC))
    fprintf (file, "playing\n");
  else
    fprintf (file, "paused\n");
#endif /* (USESOUND) */

  /* Check if no error or print it */
  if (errno)
    perror ("Faced a problem writing the options");
  
  fclose (file);
}

/* try to have real random by calling the list a few times in function of
 * the second the game is launched */
static void
core_realRand (void)
{
  int i, now;

  now = time (NULL);
  for (i = 0; i < now - (int) (now / 1000) * 1000; i++)
    rand ();
}

static void
core_printVersion (void)
{
  printf ("Jomar3D v"VERSION"\n");
}

/* Print the full help to the standard output */
static void
core_printHelp (void)
{
  core_printVersion ();

  printf ("\nUsage: jomar3d [options]\n\n"
          "Options:\n"
          "\t-nosound                    Disable all sounds\n"
          "\t-version                    Print version and exit\n"
          "\t-help                       Print this help and exit\n" "\n");

  printf ("GLUT options:\n"
          "	-display DISPLAY\n"
          "		Specify the X server to connect to.\n"
          "	-geometry WxH+X+Y\n"
          "		Determines the window's position and geometry.\n"
          "	-iconic\n"
          "		Requests the window be created in an iconic state. \n"
          "	-indirect\n"
          "		Force the use of indirect OpenGL rendering contexts. \n "
          "	-direct\n"
          "		Force the use of direct OpenGL rendering contexts.\n"
          "	-gldebug\n"
          "		After processing callbacks and/or events, check if there are any\n"
          "		OpenGL errors.\n"
          "	-sync\n"
          "		Enable synchronous X protocol transactions.\n");
}

/**  work on the command line arguments
 ** \param argc main's argc
 ** \param argv main's argv
 ** \return 0 on successn 1 on success with a exit request, other on failed */
static int
core_parseArgs (int argc, char **argv)
{
  int i;

#define CURARG (argv[i])        /* current arg */
#define ARGCMP(str) (!strcmp(CURARG, str))   /* compare current arg with str */

  for (i = 1; i < argc; i++)
  {
    if (ARGCMP ("-geometry"))
    {
      l_winuserdef = 1;
      i++;                  /* don't parse geometry */
    }
#if (USESOUND)
    else if (ARGCMP ("-nosound"))
      core_deleteState (JOMAR3D_CANMUSIC);
#endif
    else if (ARGCMP ("-version") || ARGCMP ("-V"))
    {
      core_printVersion ();
      return 1;
    }
    else if (ARGCMP ("-help") || ARGCMP ("--help") || ARGCMP ("-h"))
    {
      core_printHelp ();
      return 1;
    }
    
    debugmsg (("Argument number %d/%d: %s\n", i, argc, argv[i]));
  }

#undef ARGCMP
#undef CURARG

  return 0;
}

/* initialize and launch the game */
int
main (int argc, char **argv)
{
  int argrv;                    /* argument parser return value */
  char *envhome;

#if (USESOUND)
  core_addState (JOMAR3D_CANMUSIC);
#endif

  argrv = core_parseArgs (argc, argv);
  if (argrv != 0)
  {
    if (argrv == 1)
      return EXIT_SUCCESS;
    else
      return EXIT_FAILURE;
  }
#if (USESOUND)
  if (core_checkState(JOMAR3D_CANMUSIC))
    core_addState (JOMAR3D_PLAYMUSIC);
#endif

  core_realRand ();

/* make a filename for the preference file */
  envhome = getenv ("HOME");
  if (envhome == NULL)
    l_preffile = mystrdup ("./.jomar3Dpref");
  else
  {
    /* strlen ("/.jomar3Dpref\0") = 14 */
    l_preffile = malloc (strlen (envhome) + 14);
    if (l_preffile != NULL)
    {
      strcpy (l_preffile, envhome);
      strcat (l_preffile, "/.jomar3Dpref");
    }
  }

/* Init glut so we could check if there is a display in the sounds */
  glutInit (&argc, argv);

/* start sdl initialization */
#if (USESOUND)
  sound_launch ();
#endif


/* load and select the current language */
  core_loadLanguages ();
  core_selectLanguage ();

/* locate all available skins and select the one to use now */
  core_findSkins ();
  core_selectSkin ();

/* set the function to call at exit */
  atexit (core_cleanQuit);

/* Open the display to use and have a window (even under Linux :D) */
  display_createDisplay (l_winuserdef);

  draw_init ();
  scores_init ();

/* define the initial quality setting */
  core_initQuality ();

  glPushMatrix ();              /* push so we can pop on initial matrix */

  core_doCron (0);              /* launch the cronic jobs (self-repetitive) */
  core_deleteState (JOMAR3D_INMENU);
  interface_changeMenu (MP);

  glutMainLoop ();              /* launch all */

  return EXIT_SUCCESS;
}
