/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef CORE_H
#define CORE_H

#include <GL/gl.h>

#ifndef VERSION
# define VERSION "beta"
#endif

#ifndef SKINDIR
# define SKINDIR DATADIR"/skin"
#endif

#define NEWALL 4             /**< NEWALL: number of wells from each size (4) */
#define WALLLONG 10          /**< WALLLONG: length of the greatest wall */

/* Define a bit for each possible state */
enum
{
  JOMAR3D_INMENU         = 1<<0,     /* Whether we are in a menu */
  JOMAR3D_MENU_MAIN      = 1<<1,     /* Main menu ? */
  JOMAR3D_MENU_RULES     = 1<<2,     /* Rules menu ? */
  JOMAR3D_MENU_DISPSCORE = 1<<3,     /* Display Scores menu ? */
  JOMAR3D_MENU_OPTION    = 1<<4,     /* Option menu ? */
  JOMAR3D_MENU_FINDS     = 1<<5,     /* Finds menu ? */
  JOMAR3D_LAST           = 1<<5,     /* Is there only one Martian left ? */
  JOMAR3D_END            = 1<<6,     /* Finshed to play ? */
  JOMAR3D_SCORE          = 1<<7,     /* In the score interface ? */
  JOMAR3D_CANMUSIC       = 1<<8,     /* Is possible to play music ? */
  JOMAR3D_PLAYMUSIC      = 1<<9,     /* Is playing music ? */
  JOMAR3D_SHADING        = 1<<10,    /* Currently rotating the view ? */
  JOMAR3D_MOVING_H       = 1<<11,    /* Currently moving the view horizontally ? */
  JOMAR3D_MOVING_V       = 1<<12,    /* Currently moving the view vertically ? */
  JOMAR3D_TURN_BACK      = 1<<13     /* Do we turn the cylinder of the menu backwards ? */
};

/** 2D integer coordinate structure in map squares */
typedef struct {
  GLint x;
  GLint y;
} Coord2i;
/** 2D float coordinate structure in OpenGL units */
typedef struct {
  GLfloat x;
  GLfloat y;
} Coord2f;
/** 3D integer coordinate structure in map squares */
typedef struct {
  GLint x;
  GLint y;
  GLint z;
} Coord3i;
/** 3D float coordinate structure in OpenGL units */
typedef struct {
  GLfloat x;
  GLfloat y;
  GLfloat z;
} Coord3f;

/** Martians' data struture */
typedef struct {
  unsigned int  state;        /**< state of the Martian */
  int           speed;        /**< its speed */
  int           time;         /**< time at which it begun to move */
  float         angle;        /**< current angle */
  float         goangle;      /**< goal angle */
  Coord2i       coords;       /**< coords in squares */
  Coord2f       coordsgl;     /**< coords in GL units */
} Martian;

/** Lobster's data structure */
typedef struct {
  int     steptime;     /**< Time of a step */
  float   stepsize;     /**< Length of a step */
  int     going;
  /* Position */
  Coord2i pos;          /**< Position on the map */
  /* Lobster motion */
  float   angle;        /**< Real angle of the homard, turning counted */
  float   anglebase;    /**< Angle his direction imposes (four possibilities) */
  float   phase;        /**< Phase in its periodical walking movement */
  float   gone;         /**< Length walked from its position on the squares */
  int     basetime;     /**< Start time */
  float   legangle[4];  /**< Legs angles (by pairs) */
} Lobster;

/** Initialise staticly a Lobster struct
 * Should be used as an array initialiser:
 * Lobster lobster = LOBSTER_STATIC_INIT;
 */
#define LOBSTER_STATIC_INIT\
  { 0, 0., 0, {0, 0}, 0., 0., 0., 0., 0, {0., 0., 0., 0.} }


#define core_checkState(flags) (core_getState() & (flags))

void        core_activeMartians         (void);
void        core_computeLobsterPosition (void);
void        core_setLanguage            (int move);
void        core_setSkin                (int move);
Martian*    core_getMartian             (unsigned int number);
int         core_getMartiansLeft        (void);
int         core_getMartianSteps        (void);
Lobster*    core_getLobster             (void);
int         core_getBoostState          (void);
const char* core_getLanguageName        (void);
const char* core_getSkinName            (void);
int         core_getState               (void);
void        core_addState               (int state);
void        core_deleteState            (int state);
void        core_invertState            (int state);
const char* core_getTextPointer         (int number);
int         core_getGameDuration        (void);
void        core_initGame               (void);
#if (USESOUND)
void        core_initVolume             (void);
#endif
int         core_getLost                (void);
void        core_saveOptions            (void);
void        core_setQualityText         (const char *text);
void        core_startBoost             (void);
void        core_startMotion            (void);
void        core_stopMotion             (void);
void        core_tryEat                 (void);
void        core_wantAction             (int actiontobe,
                                         unsigned char character);


#endif /* guard */
